
/**
 * This program shows how to use FatDog XQuery implementation. It reads a file containing
 * Asterix books and prints the result based on the entered query in the command line.
 * For example: use the following queries:
 * a) //book[title&=\"Asterix The Gaul\"]
 * b) //book/title
 * Ensure that the following jar files are set in the classpath before running FatDog
 * 1) XQEngine compiled com package.
 * 2) Crimson.jar
 * 3) sax2.jar
 * 4) jaxp-api.jar
 * User: vnathan
 */

//fat dog api imports.
import com.fatdog.xmlEngine.XQEngine;
import com.fatdog.xmlEngine.query.ResultListener;
import com.fatdog.xmlEngine.exceptions.InvalidQueryException;
import com.fatdog.xmlEngine.exceptions.TypeException;
import com.fatdog.xmlEngine.exceptions.MissingOrInvalidSaxParserException;

//SaxParser imports.
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import java.io.FileNotFoundException;
//XML reader.
import org.xml.sax.XMLReader;
//Class implements Result Listener to print the XQuery results to the console.
public class FatDogEngine implements ResultListener {
    //constructor
    public FatDogEngine() {
        //do nothing.
    }
    /**
     * Method instantiates the XQEngine, constructs the SAXParsers indexes the XML document
     * and set query and prints the result.
     * @param xQueryString - XQuery string.
     * @throws FileNotFoundException
     * @throws MissingOrInvalidSaxParserException
     */
    public void runXQEngine(String xQueryString) throws FileNotFoundException, MissingOrInvalidSaxParserException {
        //construct the new XQEngine.
        XQEngine engine = new XQEngine();
        //set the listener.
        engine.setResultListener(this);
        //create the  sax parser for xml parsing.
        SAXParserFactory spf = SAXParserFactory.newInstance();
        try {
            SAXParser parser = spf.newSAXParser();
            XMLReader reader = parser.getXMLReader();

            //set the engine XML reader to the created XML reader
            engine.setXMLReader(reader);

        } catch (Exception e) {
            System.out.println("Couldn't install XMLReader");
            System.out.println(e);
        }
        engine.setDoIndexNumbers(false);
        //later set the stoplist file
       // engine.setStopList(//filepath);
        try {
            //point the engine to the XML document (indexing takes place at this point)
            engine.setDocument("D:\\asterixbooks.xml");
        } catch (Exception ef) {
            System.out.println(ef);
        }
        try {
            //set the query that is passed from the command line.
            engine.setQuery(xQueryString);
        } catch (InvalidQueryException ixe) {
            ixe.printStackTrace();
            System.err.println("Invalid Query Exception:" + ixe.getMessage());
        } catch (TypeException te) {
            te.printStackTrace();
            System.err.println("Invalid Type Exception:" + te.getMessage());
        }
    }
    /**
     * Method prints the result of the XQuery.
     * @param queryResults
     */
    public void results(String queryResults){
        System.out.println(queryResults);
    }
    //main method.
    public static void main(String args[]){
        if (args.length != 1)
            System.out.println("Usage: java FatDogEngine xQueryString");
        FatDogEngine fde = new FatDogEngine();
        try{
        fde.runXQEngine(args[0]);
        }catch(Exception e){
            System.err.println(e);
        }
     }
}
